import { system, world, EntityEquippableComponent, EquipmentSlot } from "@minecraft/server";

const tags= ["blaze", "diamond", "ender", "explosive", "glowstone", "health", "ice", "gold", "iron", "miner", "stone", "long", "knockback", "mini", "magnet", "slime", "lapis", "shulker", "sponge", "slime", "redstone", "wither", "thunder", "spider"]
const w = world.getDimension("overworld");
system.runInterval(() =>{
    try{
        for (const p of world.getPlayers()) {
            glowstoneBow(p)
            bowUsing(p)
            detectEnchantements(p)
            p.runCommandAsync("function arrow")
        }
    }catch(e){

    }
})
function glowstoneBow(p){
    p.runCommandAsync(`function glowstone_bow`)
    if (!p.hasTag("glowstone_bow")) {
         p.runCommandAsync(`function no_light`);
    }
    if (p.hasTag("glowstone_bow")) {
        p.runCommandAsync(`function light`);
    }
}
function bowUsing(p){
    const equip = p.getComponent(EntityEquippableComponent.componentId);
    const hand = equip.getEquipment(EquipmentSlot.Mainhand);
    for(const bows of tags){
        if(p.hasTag("using")){
            if(hand.typeId == `new:${bows}_bow`){
                p.runCommandAsync(`playanimation @s[hasitem={item=new:${bows}_arrow}] animation.weapons.bow_and_arrow root 0.001 "!query.is_using_item"`)
            }
        }
    }
}

function detectEnchantements(p){
    const equip = p.getComponent(EntityEquippableComponent.componentId);
    const hand = equip.getEquipment(EquipmentSlot.Mainhand);
    if(hand){ 
        const enchantable = hand.getComponent("enchantable");
        for(const bows of tags){
            if(hand.typeId.includes(`${bows}_bow`)){
                if (enchantable) {
                    if (enchantable.getEnchantment("infinity")) {
                        p.addTag(`enchanted:infinity`)
                    }
                    if (!enchantable.getEnchantment("infinity")) {
                        p.removeTag(`enchanted:infinity`)
                    }
                    if(enchantable.getEnchantments().length === 0){
                        p.getTags().forEach(tags => {
                            if(tags.includes("enchanted:")){
                                p.removeTag(tags)
                            }
                        });
                    }
                }
            }
        }
        if(!hand.typeId.includes(`_bow`)){
            p.getTags().forEach(tag => {
                if(tag.includes("enchanted:")){
                    p.removeTag(tag)
                }
            });
        }
    }
    if (!hand) {
        p.getTags().forEach(tag => {
            if(tag.includes("enchanted:")){
                p.removeTag(tag)
            }
        });
    }
}

world.afterEvents.itemUse.subscribe((use) =>{
    try{
        const p = use.source;
        const bow = "_bow";
        const item = use.itemStack.typeId;
        if(item.toLowerCase().includes(bow)){
            p.addTag("using");
        } 
    }catch(e){

    }
});

world.afterEvents.playerSpawn.subscribe(spawn=>{
    world.gameRules.sendCommandFeedback=false
})

world.afterEvents.itemStopUse.subscribe((stop) =>{
    try{
        const p = stop.source;
        const item = stop.itemStack;
        p.removeTag("using");
        for(const bows of tags){
            if(p.hasTag(`${bows}`)){
                if(item.typeId == `new:${bows}_bow` && p.hasTag("using")){
                    Durability(item, p)
                }
            }
        }
    }catch(e){

    }
})

world.afterEvents.projectileHitEntity.subscribe(hit=>{
    try{
        const p = hit.source;
        const projectile = hit.projectile;
        const target = hit.getEntityHit()
        if(projectile && target){
            for(const bows of tags){
                if(projectile.typeId.includes(`new:arrow_${bows}`)){
                    if(p.hasTag(`enchanted:infinity`)){
                        projectile.remove()
                    }
                }
            }
        }
    }catch(e){

    }
})

world.afterEvents.projectileHitBlock.subscribe(hit=>{
    try{
        const p = hit.source;
        const projectile = hit.projectile;
        const target = hit.getBlockHit()
        if(projectile && target){
            for(const bows of tags){
                if(projectile.typeId.includes(`new:arrow_${bows}`)){
                    if(p.hasTag(`enchanted:infinity`)){
                        projectile.remove()
                    }
                }
            }
        }
    }catch(e){

    }
})

export function Durability(tool, source) {
	try{
		system.run(()=>{
			const component = source.getComponent(EntityEquippableComponent.componentId);
			const durability = tool.getComponent("durability");
            const enchantable = tool.getComponent("enchantable");
			if(tool && durability !== undefined){
                let unbreakingLevel = 0;
                const damageAmount = 2;
                if (enchantable) {
                    for (const enchant of enchantable.getEnchantments()) {
                        if (enchant.type.id === "unbreaking") {
                            unbreakingLevel = enchant.level;
                            break;
                        }
                    }
                }if(durability.damage+2 < durability.maxDurability){
					if(unbreakingLevel > 0) {
                        const chance = 1 / (unbreakingLevel + 1);
                        if (Math.random() < chance) {
                            durability.damage += damageAmount;
                        }
                    } if(unbreakingLevel == 0) {
                        durability.damage += damageAmount;
                    }
					component.setEquipment(EquipmentSlot.Mainhand, tool)
				}else if(durability.damage+2 >= durability.maxDurability){
					const air = new ItemStack("minecraft:air")
					component.setEquipment(EquipmentSlot.Mainhand, air)
					source.playSound("random.break")
				}
			}
		})
	}catch(e){}
}